% #########################################################################
% flim_example_file_script.m
% #########################################################################

% This MATLAB script demonstrates the use of FLIM functions to obtain
% lifetime images out of image stacks stored in multi-TIFF files.

% #########################################################################
% settings
% #########################################################################

% input files
refFileName = './data/reference_3.55ns_30MHz_2x8.tif'; % reference
sampleFileName = './data/sample_30MHz_2x8.tif';        % sample

% measurement parameters
f = 30E6;         % modulation frequency (in Hertz)
tauRef = 3.55E-9; % lifetime of reference (in seconds)
nPhases = 8;      % number of equidistant phases per modulation period

% raw image stack properties
NUM_BITS = 14;
BIT_ALIGN_DIV = 4; % 14 bit data MSB aligned in 16 bit TIFF file

phaseNumber = '08';       % 8 equidistant phases per modulation period
addPhaseSampling = 'yes'; % both taps A and B hold all phase information
phaseOrder = 'ascending'; % ascending order of tap A and B phases
tapSelection = 'Tap A+B'; % both taps A and B are read out
asymCorrection = 'off';   % no asymmetry correction inside the camera

% display properties
dispMinLifetime = 0.5E-9; % minimum lifetime (in seconds)
dispMaxLifetime = 3.5E-9; % maximum lifetime (in seconds)

% #########################################################################
% call example function
% #########################################################################

flim_example_file(refFileName, sampleFileName, f, tauRef, ...
    nPhases, NUM_BITS, BIT_ALIGN_DIV, phaseNumber, addPhaseSampling, ...
    phaseOrder, tapSelection, asymCorrection, dispMinLifetime, ...
    dispMaxLifetime);
